@@ -29,7 +29,46 @@
 #include "triedefs.h"
 
 #include <string.h>
+#include <stdlib.h>
 
+/*----------------------------*
+ *   STATIC TRIECHAR STRING   *
+ *----------------------------*/
+
+size_t
+trie_char_strlen (const TrieChar *str)
+{
+    size_t len = 0;
+    while (*str++ != TRIE_CHAR_TERM) {
+        ++len;
+    }
+    return len;
+}
+
+size_t
+trie_char_strsize (const TrieChar *str)
+{
+    return trie_char_strlen (str) * sizeof (TrieChar);
+}
+
+TrieChar *
+trie_char_strdup (const TrieChar *str)
+{
+    TrieChar *dup
+        = (TrieChar *) malloc (sizeof (TrieChar) * (trie_char_strlen (str) + 1));
+    TrieChar *p = dup;
+
+    while (*str != TRIE_CHAR_TERM) {
+        *p++ = *str++;
+    }
+    *p = TRIE_CHAR_TERM;
+
+    return dup;
+}
+
+/*-----------------------------*
+ *   DYNAMIC TRIECHAR STRING   *
+ *-----------------------------*/
 
 struct _TrieString {
     DString ds;
@@ -100,7 +139,8 @@ trie_string_append_char (TrieString *ts, TrieChar tc)
 Bool
 trie_string_terminate (TrieString *ts)
 {
-    return dstring_terminate ((DString *)ts);
+    static const TrieChar term = TRIE_CHAR_TERM;
+    return dstring_append_char ((DString *)ts, &term);
 }
 
 Bool
